"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("../types");
var DefinitionMetadata_1 = require("./DefinitionMetadata");
/**
 * Keep all [[DefinitionMetadata]] and [[GenericMetadata]].
 * Provide methods to query metadata with its associated class or graphql type name.
 */
var MetadataStorage = /** @class */ (function () {
    function MetadataStorage() {
        this.genericMetadata = [];
        this.definitionMetadata = [];
    }
    /**
     * Add a new [[Metadata]] object to storage.
     * @param metadata A metadata object to register
     */
    MetadataStorage.prototype.register = function (metadata) {
        if (metadata instanceof DefinitionMetadata_1.DefinitionMetadata) {
            this.definitionMetadata.unshift(metadata);
        }
        else {
            this.genericMetadata.push(metadata);
        }
    };
    /**
     * Get a [[DefinitionMetadata]] object which is instance of the `metadataClass` and associated to `definitionClass`
     * @param metadataClass A [[DefinitionMetadata]] subclass to query
     * @param definitionClass A class associated with metadata to query
     */
    MetadataStorage.prototype.getDefinitionMetadata = function (metadataClass, definitionClass) {
        var foundMetadata = this.definitionMetadata.find(function (metadata) {
            return types_1.isSubClassOf(definitionClass, metadata.definitionClass) && metadata instanceof metadataClass;
        });
        if (!foundMetadata) {
            throw new Error("Cannot get " + metadataClass.name + " of " + definitionClass.name + " from MetadataStorage");
        }
        return foundMetadata;
    };
    /**
     * Get a [[DefinitionMetadata]] object by the name of the GraphQLType that the metadata will generate.
     * @param name The name of a GraphQLType which will be built from metadata to query
     */
    MetadataStorage.prototype.getDefinitionMetadataByName = function (name) {
        var metadata = this.definitionMetadata.find(function (meta) { return meta.typeName === name; });
        if (!metadata) {
            throw new Error("Cannot find metadata with given name: " + name);
        }
        return metadata;
    };
    /**
     * Find all [[GenericMetadata]] which is instance of the `metadataClass` and associated to `definitionClass`
     * @param metadataClass A [[GenericMetadata]] subclass to query
     * @param definitionClass A class associated with metadata to query
     */
    MetadataStorage.prototype.findGenericMetadata = function (metadataClass, definitionClass) {
        var filterdMetadata = this.genericMetadata.filter(function (metadata) {
            return types_1.isSubClassOf(definitionClass, metadata.definitionClass) && metadata instanceof metadataClass;
        });
        return filterdMetadata;
    };
    return MetadataStorage;
}());
exports.MetadataStorage = MetadataStorage;
//# sourceMappingURL=MetadataStorage.js.map