"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TypeExpression_1 = require("../type-expression/TypeExpression");
var ObjectTypeMetadata_1 = require("../metadata/ObjectTypeMetadata");
var FieldMetadata_1 = require("../metadata/FieldMetadata");
var ArgumentMetadata_1 = require("../metadata/ArgumentMetadata");
var InputFieldMetadata_1 = require("../metadata/InputFieldMetadata");
var structure_1 = require("../type-expression/structure");
var InterfaceTypeMetadata_1 = require("../metadata/InterfaceTypeMetadata");
var InputObjectTypeMetadata_1 = require("../metadata/InputObjectTypeMetadata");
function createMetadataFromAST(node, definitionClass, substitutionMap) {
    if (node.kind === 'ObjectTypeDefinition') {
        return createObjectTypeMetadataFromAST(node, definitionClass, substitutionMap);
    }
    else if (node.kind === 'InterfaceTypeDefinition') {
        return createInterfaceTypeMetadataFromAST(node, definitionClass, substitutionMap);
    }
    else if (node.kind === 'InputObjectTypeDefinition') {
        return createInputObjectTypeMetadataFromAST(node, definitionClass, substitutionMap);
    }
    else {
        throw new Error("Node type not supported: " + node.kind + "(" + definitionClass + ")");
    }
}
exports.createMetadataFromAST = createMetadataFromAST;
function createTypeExpressionFromAST(type, substitutionMap) {
    if (type.kind === 'ListType') {
        return structure_1.List.of(createTypeExpressionFromAST(type.type, substitutionMap));
    }
    else if (type.kind === 'NonNullType') {
        return structure_1.NonNull.of(createTypeExpressionFromAST(type.type, substitutionMap));
    }
    else {
        var givenExpression = substitutionMap[type.name.value];
        if (!givenExpression) {
            return new TypeExpression_1.TypeExpression(type.name.value);
        }
        else if (givenExpression instanceof TypeExpression_1.TypeExpression) {
            return givenExpression;
        }
        return new TypeExpression_1.TypeExpression(givenExpression);
    }
}
exports.createTypeExpressionFromAST = createTypeExpressionFromAST;
function createObjectTypeMetadataFromAST(node, definitionClass, substitutionMap) {
    var name = node.name, description = node.description, interfaces = node.interfaces;
    var generics = node.fields.reduce(function (results, fieldNode) { return (results.concat(createFieldMetadataFromAST(fieldNode, definitionClass, substitutionMap))); }, []);
    var interfacesTypeExpressions = interfaces && interfaces.map(function (interfaceNode) { return (createTypeExpressionFromAST(interfaceNode, substitutionMap)); });
    var definition = new ObjectTypeMetadata_1.ObjectTypeMetadata(definitionClass, {
        interfaces: interfacesTypeExpressions,
        typeName: name.value,
        description: description && description.value,
    });
    return { definition: definition, generics: generics };
}
exports.createObjectTypeMetadataFromAST = createObjectTypeMetadataFromAST;
function createInterfaceTypeMetadataFromAST(node, definitionClass, substitutionMap) {
    var name = node.name, description = node.description;
    var generics = node.fields.reduce(function (results, fieldNode) { return (results.concat(createFieldMetadataFromAST(fieldNode, definitionClass, substitutionMap))); }, []);
    var definition = new InterfaceTypeMetadata_1.InterfaceTypeMetadata(definitionClass, {
        typeName: name.value,
        description: description && description.value,
    });
    return { definition: definition, generics: generics };
}
exports.createInterfaceTypeMetadataFromAST = createInterfaceTypeMetadataFromAST;
function createInputObjectTypeMetadataFromAST(node, definitionClass, substitutionMap) {
    var name = node.name, description = node.description;
    var generics = node.fields.reduce(function (results, fieldNode) { return (results.concat(createInputFieldMetadataFromAST(fieldNode, definitionClass, substitutionMap))); }, []);
    var definition = new InputObjectTypeMetadata_1.InputObjectTypeMetadata(definitionClass, {
        typeName: name.value,
        description: description && description.value,
    });
    return { definition: definition, generics: generics };
}
exports.createInputObjectTypeMetadataFromAST = createInputObjectTypeMetadataFromAST;
function createFieldMetadataFromAST(node, definitionClass, substitutionMap) {
    var name = node.name, type = node.type, description = node.description;
    var argumentMetadata = node.arguments.map(function (argumentNode) { return (createArgumentMetadataFromAST(argumentNode, definitionClass, name.value, substitutionMap)); });
    // static method to resolver
    var descriptor = Object.getOwnPropertyDescriptor(definitionClass, name.value);
    var describedFunction = descriptor && (descriptor.value || descriptor.get);
    var resolver = describedFunction instanceof Function
        ? describedFunction
        : undefined;
    var fieldMetadata = new FieldMetadata_1.FieldMetadata(definitionClass, {
        typeExpression: createTypeExpressionFromAST(type, substitutionMap),
        resolver: resolver,
        fieldName: name.value,
        description: description && description.value,
    });
    return [fieldMetadata].concat(argumentMetadata);
}
function createArgumentMetadataFromAST(node, definitionClass, fieldName, substitutionMap) {
    var name = node.name, type = node.type, description = node.description;
    return new ArgumentMetadata_1.ArgumentMetadata(definitionClass, {
        fieldName: fieldName,
        description: description && description.value,
        argumentName: name.value,
        typeExpression: createTypeExpressionFromAST(type, substitutionMap),
    });
}
function createInputFieldMetadataFromAST(node, definitionClass, substitutionMap) {
    var name = node.name, type = node.type, description = node.description;
    return new InputFieldMetadata_1.InputFieldMetadata(definitionClass, {
        fieldName: name.value,
        description: description && description.value,
        typeExpression: createTypeExpressionFromAST(type, substitutionMap),
    });
}
//# sourceMappingURL=ast-to-metadata.js.map