"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var structure_1 = require("../type-expression/structure");
var directive_1 = require("./directive");
var TypeExpression_1 = require("../type-expression/TypeExpression");
function gql(strings) {
    var typeArgs = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        typeArgs[_i - 1] = arguments[_i];
    }
    var result = [];
    var substitution_prefix = '__girin__intermediate__';
    result.push(strings[0]);
    for (var i = 0; i < strings.length - 1; i++) {
        result.push("" + substitution_prefix + i);
        result.push(strings[i + 1]);
    }
    var rootNode = graphql_1.parse(result.join('')).definitions[0];
    var substitutionMap = typeArgs.reduce(function (results, exp, i) {
        var name = "" + substitution_prefix + i;
        results[name] = exp;
        return results;
    }, {});
    return new ASTParser(rootNode, substitutionMap);
}
exports.gql = gql;
var ASTParser = /** @class */ (function () {
    function ASTParser(rootNode, substitutionMap) {
        this.objectTypeMetadataConfigs = [];
        this.interfaceTypeMetadataConfigs = [];
        this.inputObjectTypeMetadataConfigs = [];
        this.fieldMetadataConfigs = [];
        this.argumentMetadataConfigs = [];
        this.inputFieldMetadataConfigs = [];
        this.rootNode = rootNode;
        this.substitutionMap = substitutionMap;
        this.createMetadataFromAST();
    }
    ASTParser.prototype.createMetadataFromAST = function () {
        var rootNode = this.rootNode;
        if (rootNode.kind === 'ObjectTypeDefinition') {
            this.appendObjectTypeConfig(rootNode);
        }
        else if (rootNode.kind === 'InterfaceTypeDefinition') {
            this.appendInterfaceTypeMetadataConfig(rootNode);
        }
        else if (rootNode.kind === 'InputObjectTypeDefinition') {
            this.appendInputObjectTypeMetadataConfig(rootNode);
        }
        else {
            throw new Error("Node type not supported: " + rootNode.kind);
        }
    };
    ASTParser.prototype.completeTypeExpression = function (type) {
        var substitutionMap = this.substitutionMap;
        if (type.kind === 'ListType') {
            return structure_1.List.of(this.completeTypeExpression(type.type));
        }
        else if (type.kind === 'NonNullType') {
            return structure_1.NonNull.of(this.completeTypeExpression(type.type));
        }
        else {
            var givenExpression = substitutionMap[type.name.value];
            if (!givenExpression) {
                return new TypeExpression_1.TypeExpression(type.name.value);
            }
            else if (givenExpression instanceof TypeExpression_1.TypeExpression) {
                return givenExpression;
            }
            return new TypeExpression_1.TypeExpression(givenExpression);
        }
    };
    ASTParser.prototype.appendObjectTypeConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description, interfaces = node.interfaces;
        node.fields.forEach(function (fieldNode) { return _this.appendFieldMetadataConfig(fieldNode); });
        var interfacesTypeExpressions = interfaces && interfaces.map(function (interfaceNode) { return (_this.completeTypeExpression(interfaceNode)); });
        this.objectTypeMetadataConfigs.push({
            interfaces: interfacesTypeExpressions,
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendInterfaceTypeMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description;
        node.fields.forEach(function (fieldNode) { return _this.appendFieldMetadataConfig(fieldNode); });
        this.interfaceTypeMetadataConfigs.push({
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendInputObjectTypeMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, description = node.description;
        node.fields.forEach(function (fieldNode) { return _this.appendInputFieldMetadataConfig(fieldNode); });
        this.inputObjectTypeMetadataConfigs.push({
            typeName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendFieldMetadataConfig = function (node) {
        var _this = this;
        var name = node.name, type = node.type, description = node.description;
        node.arguments.forEach(function (argumentNode) { return (_this.appendArgumentMetadataConfig(argumentNode, name.value)); });
        this.fieldMetadataConfigs.push({
            typeExpression: this.completeTypeExpression(type),
            fieldName: name.value,
            description: description && description.value,
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendArgumentMetadataConfig = function (node, fieldName) {
        var name = node.name, type = node.type, description = node.description;
        this.argumentMetadataConfigs.push({
            fieldName: fieldName,
            description: description && description.value,
            argumentName: name.value,
            typeExpression: this.completeTypeExpression(type),
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    ASTParser.prototype.appendInputFieldMetadataConfig = function (node) {
        var name = node.name, type = node.type, description = node.description;
        this.inputFieldMetadataConfigs.push({
            fieldName: name.value,
            description: description && description.value,
            typeExpression: this.completeTypeExpression(type),
            directives: node.directives && directive_1.completeDirectives(node.directives),
        });
    };
    return ASTParser;
}());
exports.ASTParser = ASTParser;
//# sourceMappingURL=ast.js.map